% Introduction to finite elements and sparse linear system solving
% Fall 2017 Exercise 4-3

% The system PDE problem 
% -nabla (c \nabla u) + a u = f 
% Our problem: -Delta u = 1 ==> c=1, a=0, 
g='squareg';
b='squareb1';
c=1; a=0;
f= '2*(x.^3-x)-6*(1-y.^2).*x';

hp=[];
err=[];
n=4;
er = 1;

while er>1e-2

   [p,e,t]=poimesh(g,n); %generate a regular mesh

   [A2,~,f2]=assema(p,t,c,a,f);  %assemble the problem

   nn = size(p,2);               % number of nodes

   dirichlet = unique([e(1,:) e(2,:)]);  % determine boundary points
   interior = setdiff([1:nn],dirichlet);
   A2 = A2(interior,interior);           % extract interior points
   f2 = f2(interior);

   u2 = A2\f2;  %solve the system	  

   %compute the error
   %Energy of the system is 3328/1575
   energy2 = u2'*A2*u2;                % discrete energy
   er = sqrt((3328/1575-energy2));     % energy error
   err = [err,er];
   hp = [hp,2/n];

   %prepare next step
   n = n*2;

end

x=0:0.1:1;
plot(hp,err,x,x,x,x.^2);
legend ('Energy error','x','x^2')
set(gca,'XDir','reverse','XScale','log','YScale','log');
