% Introduction to finite elements and sparse linear system solving
% Fall 2017 Exercise 4-4

% The system PDE problem 
% -nabla (c \nabla u) + a u = f 
% Our problem: -Delta u = 1 ==> c=1, a=0, f = 1
g = 'squareg'; %% 
g = 'lshapeg';
c = 1; a = 0; f = 1;

hp = [];
err = [];
n = 2; 
er = 1;

[p,e,t] = initmesh(g,'hmax',1/n); %generate a mesh

while er>1e-2

   [A2,~,f2] = assema(p,t,c,a,f);  %assemble the problem

   nn = size(p,2);               % number of nodes

   dirichlet = unique([e(1,e(6,:)==0), e(2,e(6,:)==0) e(1,e(7,:)==0), ...
                       e(2,e(7,:)==0)]);     % determine boundary points
   interior = setdiff([1:nn],dirichlet);

   A2 = A2(interior,interior);           % extract interior points
   f2 = f2(interior);

   u2 = A2\f2;  %solve the system	  

   u=zeros(nn,1); u(interior) = u2;

   %compute the error
   %Energy of the system is 3328/1575
   energy2 = u2'*A2*u2;                % discrete energy
   er = sqrt(0.214069970362001-energy2);     % energy error
   err = [err,er];
   hp = [hp,2/n];

   %prepare next step
   n = n*2;
   [p,e,t] = refinemesh(g,p,e,t);
   p = jigglemesh(p,e,t);

end
fprintf('\nNumber of nodes = %u, number of elements = %u\n',nn,size(t,2))

x = 0:0.1:1;
plot(hp,err,x,x*err(1)*0.9,x,x.^(2/3)*err(1)*1.1);
legend ('Energy error','x','x^{2/3}')
set(gca,'XDir','reverse','XScale','log','YScale','log');

